@@ -46,6 +46,8 @@ GST_DEBUG_CATEGORY_EXTERN (GST_CAT_PERFORMANCE);
 #define DEFAULT_DIRECT_RENDERING	TRUE
 #define DEFAULT_DEBUG_MV		FALSE
 #define DEFAULT_MAX_THREADS		0
+#define DEFAULT_OUTPUT_CORRUPT		TRUE
+#define REQUIRED_POOL_MAX_BUFFERS       32
 
 enum
 {
@@ -55,6 +57,7 @@ enum
   PROP_DIRECT_RENDERING,
   PROP_DEBUG_MV,
   PROP_MAX_THREADS,
+  PROP_OUTPUT_CORRUPT,
   PROP_LAST
 };
 
@@ -223,6 +226,10 @@ gst_ffmpegviddec_class_init (GstFFMpegVidDecClass * klass)
       g_param_spec_boolean ("debug-mv", "Debug motion vectors",
           "Whether libav should print motion vectors on top of the image",
           DEFAULT_DEBUG_MV, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+  g_object_class_install_property (gobject_class, PROP_OUTPUT_CORRUPT,
+      g_param_spec_boolean ("output-corrupt", "Output corrupt buffers",
+          "Whether libav should output frames even if corrupted",
+          DEFAULT_OUTPUT_CORRUPT, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   caps = klass->in_plugin->capabilities;
   if (caps & (CODEC_CAP_FRAME_THREADS | CODEC_CAP_SLICE_THREADS)) {
@@ -258,6 +265,9 @@ gst_ffmpegviddec_init (GstFFMpegVidDec * ffmpegdec)
   ffmpegdec->direct_rendering = DEFAULT_DIRECT_RENDERING;
   ffmpegdec->debug_mv = DEFAULT_DEBUG_MV;
   ffmpegdec->max_threads = DEFAULT_MAX_THREADS;
+  ffmpegdec->output_corrupt = DEFAULT_OUTPUT_CORRUPT;
+
+  gst_video_decoder_set_needs_format (GST_VIDEO_DECODER (ffmpegdec), TRUE);
 }
 
 static void
@@ -276,6 +286,17 @@ gst_ffmpegviddec_finalize (GObject * object)
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
+static void
+gst_ffmpegviddec_context_set_flags (AVCodecContext * context, guint flags,
+    gboolean enable)
+{
+  g_return_if_fail (context != NULL);
+
+  if (enable)
+    context->flags |= flags;
+  else
+    context->flags &= ~flags;
+}
 
 /* with LOCK */
 static gboolean
@@ -347,6 +368,9 @@ gst_ffmpegviddec_open (GstFFMpegVidDec * ffmpegdec)
       break;
   }
 
+  gst_ffmpegviddec_context_set_flags (ffmpegdec->context,
+      CODEC_FLAG_OUTPUT_CORRUPT, ffmpegdec->output_corrupt);
+
   return TRUE;
 
   /* ERRORS */
@@ -523,6 +547,7 @@ open_failed:
 
 typedef struct
 {
+  GstFFMpegVidDec *ffmpegdec;
   GstVideoCodecFrame *frame;
   gboolean mapped;
   GstVideoFrame vframe;
@@ -530,13 +555,17 @@ typedef struct
 } GstFFMpegVidDecVideoFrame;
 
 static GstFFMpegVidDecVideoFrame *
-gst_ffmpegviddec_video_frame_new (GstVideoCodecFrame * frame)
+gst_ffmpegviddec_video_frame_new (GstFFMpegVidDec * ffmpegdec,
+    GstVideoCodecFrame * frame)
 {
   GstFFMpegVidDecVideoFrame *dframe;
 
   dframe = g_slice_new0 (GstFFMpegVidDecVideoFrame);
+  dframe->ffmpegdec = ffmpegdec;
   dframe->frame = frame;
 
+  GST_DEBUG_OBJECT (ffmpegdec, "new video frame %p", dframe);
+
   return dframe;
 }
 
@@ -544,6 +573,8 @@ static void
 gst_ffmpegviddec_video_frame_free (GstFFMpegVidDec * ffmpegdec,
     GstFFMpegVidDecVideoFrame * frame)
 {
+  GST_DEBUG_OBJECT (ffmpegdec, "free video frame %p", frame);
+
   if (frame->mapped)
     gst_video_frame_unmap (&frame->vframe);
   gst_video_decoder_release_frame (GST_VIDEO_DECODER (ffmpegdec), frame->frame);
@@ -551,6 +582,14 @@ gst_ffmpegviddec_video_frame_free (GstFFMpegVidDec * ffmpegdec,
   g_slice_free (GstFFMpegVidDecVideoFrame, frame);
 }
 
+static void
+dummy_free_buffer (void *opaque, uint8_t * data)
+{
+  GstFFMpegVidDecVideoFrame *frame = opaque;
+
+  gst_ffmpegviddec_video_frame_free (frame->ffmpegdec, frame);
+}
+
 /* called when ffmpeg wants us to allocate a buffer to write the decoded frame
  * into. We try to give it memory from our pool */
 static int
@@ -589,9 +628,8 @@ gst_ffmpegviddec_get_buffer (AVCodecContext * context, AVFrame * picture)
     goto duplicate_frame;
 
   /* GstFFMpegVidDecVideoFrame receives the frame ref */
-  picture->opaque = dframe = gst_ffmpegviddec_video_frame_new (frame);
-
-  ffmpegdec->opaques = g_slist_prepend (ffmpegdec->opaques, dframe);
+  picture->opaque = dframe =
+      gst_ffmpegviddec_video_frame_new (ffmpegdec, frame);
 
   GST_DEBUG_OBJECT (ffmpegdec, "storing opaque %p", dframe);
 
@@ -695,12 +733,19 @@ invalid_frame:
 fallback:
   {
     int c;
+    gboolean first = TRUE;
     int ret = avcodec_default_get_buffer (context, picture);
 
-    GST_LOG_OBJECT (ffmpegdec, "performing fallback alloc");
-    for (c = 0; c < AV_NUM_DATA_POINTERS; c++)
+    for (c = 0; c < AV_NUM_DATA_POINTERS; c++) {
       ffmpegdec->stride[c] = picture->linesize[c];
 
+      if (picture->buf[c] == NULL && first) {
+        picture->buf[c] =
+            av_buffer_create (NULL, 0, dummy_free_buffer, dframe, 0);
+        first = FALSE;
+      }
+    }
+
     return ret;
   }
 duplicate_frame:
@@ -778,8 +823,8 @@ gst_ffmpegviddec_release_buffer (AVCodecContext * context, AVFrame * picture)
 
   ffmpegdec = (GstFFMpegVidDec *) context->opaque;
   frame = (GstFFMpegVidDecVideoFrame *) picture->opaque;
-  GST_DEBUG_OBJECT (ffmpegdec, "release frame SN %d (%p)",
-      frame->frame->system_frame_number, frame);
+  GST_DEBUG_OBJECT (ffmpegdec, "release frame SN %d",
+      frame->frame->system_frame_number);
 
   /* check if it was our buffer */
   if (picture->type != FF_BUFFER_TYPE_USER) {
@@ -792,8 +837,6 @@ gst_ffmpegviddec_release_buffer (AVCodecContext * context, AVFrame * picture)
 
   gst_ffmpegviddec_video_frame_free (ffmpegdec, frame);
 
-  ffmpegdec->opaques = g_slist_remove (ffmpegdec->opaques, frame);
-
   /* zero out the reference in ffmpeg */
   for (i = 0; i < 4; i++) {
     picture->data[i] = NULL;
@@ -943,6 +986,23 @@ gst_ffmpegviddec_negotiate (GstFFMpegVidDec * ffmpegdec,
   else
     out_info->interlace_mode = GST_VIDEO_INTERLACE_MODE_PROGRESSIVE;
 
+  switch (context->chroma_sample_location) {
+    case 1:
+      out_info->chroma_site = GST_VIDEO_CHROMA_SITE_MPEG2;
+      break;
+    case 2:
+      out_info->chroma_site = GST_VIDEO_CHROMA_SITE_JPEG;
+      break;
+    case 3:
+      out_info->chroma_site = GST_VIDEO_CHROMA_SITE_DV;
+      break;
+    case 4:
+      out_info->chroma_site = GST_VIDEO_CHROMA_SITE_V_COSITED;
+      break;
+    default:
+      break;
+  }
+
   /* try to find a good framerate */
   if ((in_info->fps_d && in_info->fps_n) ||
       GST_VIDEO_INFO_FLAG_IS_SET (in_info, GST_VIDEO_FLAG_VARIABLE_FPS)) {
@@ -1006,10 +1066,8 @@ negotiate_failed:
  * Sets the skip_frame flag and if things are really bad, skips to the next
  * keyframe.
  *
- * Returns TRUE if the frame should be decoded, FALSE if the frame can be dropped
- * entirely.
  */
-static gboolean
+static void
 gst_ffmpegviddec_do_qos (GstFFMpegVidDec * ffmpegdec,
     GstVideoCodecFrame * frame, gboolean * mode_switch)
 {
@@ -1018,47 +1076,29 @@ gst_ffmpegviddec_do_qos (GstFFMpegVidDec * ffmpegdec,
   *mode_switch = FALSE;
 
   if (frame == NULL)
-    goto no_qos;
+    return;
 
   diff =
       gst_video_decoder_get_max_decode_time (GST_VIDEO_DECODER (ffmpegdec),
       frame);
 
   /* if we don't have timing info, then we don't do QoS */
-  if (G_UNLIKELY (!GST_CLOCK_TIME_IS_VALID (diff)))
-    goto no_qos;
+  if (G_UNLIKELY (diff == G_MAXINT64))
+    return;
 
-  GST_DEBUG_OBJECT (ffmpegdec, "decoding time %" GST_TIME_FORMAT,
-      GST_TIME_ARGS (diff));
+  GST_DEBUG_OBJECT (ffmpegdec, "decoding time %" G_GINT64_FORMAT, diff);
 
-  if (diff > 0)
-    goto normal_mode;
-
-  if (diff <= 0) {
-    goto skip_frame;
+  if (diff > 0 && ffmpegdec->context->skip_frame != AVDISCARD_DEFAULT) {
+    ffmpegdec->context->skip_frame = AVDISCARD_DEFAULT;
+    *mode_switch = TRUE;
+    GST_DEBUG_OBJECT (ffmpegdec, "QOS: normal mode");
   }
 
-no_qos:
-  return TRUE;
-
-normal_mode:
-  {
-    if (ffmpegdec->context->skip_frame != AVDISCARD_DEFAULT) {
-      ffmpegdec->context->skip_frame = AVDISCARD_DEFAULT;
-      *mode_switch = TRUE;
-      GST_DEBUG_OBJECT (ffmpegdec, "QOS: normal mode");
-    }
-    return TRUE;
-  }
-skip_frame:
-  {
-    if (ffmpegdec->context->skip_frame != AVDISCARD_NONREF) {
-      ffmpegdec->context->skip_frame = AVDISCARD_NONREF;
-      *mode_switch = TRUE;
-      GST_DEBUG_OBJECT (ffmpegdec,
-          "QOS: hurry up, diff %" G_GINT64_FORMAT " >= 0", diff);
-    }
-    return FALSE;
+  else if (diff <= 0 && ffmpegdec->context->skip_frame != AVDISCARD_NONREF) {
+    ffmpegdec->context->skip_frame = AVDISCARD_NONREF;
+    *mode_switch = TRUE;
+    GST_DEBUG_OBJECT (ffmpegdec,
+        "QOS: hurry up, diff %" G_GINT64_FORMAT " >= 0", diff);
   }
 }
 
@@ -1141,13 +1181,11 @@ gst_avpacket_init (AVPacket * packet, guint8 * data, guint size)
  */
 static gint
 gst_ffmpegviddec_video_frame (GstFFMpegVidDec * ffmpegdec,
-    guint8 * data, guint size, GstVideoCodecFrame * frame, GstFlowReturn * ret)
+    guint8 * data, guint size, gint * have_data, GstVideoCodecFrame * frame,
+    GstFlowReturn * ret)
 {
   gint len = -1;
-  gint have_data;
   gboolean mode_switch;
-  gboolean decode;
-  gint skip_frame = AVDISCARD_DEFAULT;
   GstVideoCodecFrame *out_frame;
   GstFFMpegVidDecVideoFrame *out_dframe;
   AVPacket packet;
@@ -1159,7 +1197,7 @@ gst_ffmpegviddec_video_frame (GstFFMpegVidDec * ffmpegdec,
 
   /* run QoS code, we don't stop decoding the frame when we are late because
    * else we might skip a reference frame */
-  decode = gst_ffmpegviddec_do_qos (ffmpegdec, frame, &mode_switch);
+  gst_ffmpegviddec_do_qos (ffmpegdec, frame, &mode_switch);
 
   if (ffmpegdec->is_realvideo && data != NULL) {
     gint slice_count;
@@ -1179,13 +1217,6 @@ gst_ffmpegviddec_video_frame (GstFFMpegVidDec * ffmpegdec,
     }
   }
 
-  if (!decode) {
-    /* no decoding needed, save previous skip_frame value and brutely skip
-     * decoding everything */
-    skip_frame = ffmpegdec->context->skip_frame;
-    ffmpegdec->context->skip_frame = AVDISCARD_NONREF;
-  }
-
   if (frame) {
     /* save reference to the timing info */
     ffmpegdec->context->reordered_opaque = (gint64) frame->system_frame_number;
@@ -1208,14 +1239,10 @@ gst_ffmpegviddec_video_frame (GstFFMpegVidDec * ffmpegdec,
   }
 
   len = avcodec_decode_video2 (ffmpegdec->context,
-      ffmpegdec->picture, &have_data, &packet);
-
-  /* restore previous state */
-  if (!decode)
-    ffmpegdec->context->skip_frame = skip_frame;
+      ffmpegdec->picture, have_data, &packet);
 
   GST_DEBUG_OBJECT (ffmpegdec, "after decode: len %d, have_data %d",
-      len, have_data);
+      len, *have_data);
 
   /* when we are in skip_frame mode, don't complain when ffmpeg returned
    * no data because we told it to skip stuff. */
@@ -1223,17 +1250,9 @@ gst_ffmpegviddec_video_frame (GstFFMpegVidDec * ffmpegdec,
     len = 0;
 
   /* no data, we're done */
-  if (len < 0 || have_data <= 0)
+  if (len < 0 || *have_data == 0)
     goto beach;
 
-  GST_LOG_OBJECT (ffmpegdec, "picture opaque %p", ffmpegdec->picture->opaque);
-  /* libav might be tripping, and handing a picture with invalid opaque
-   * (e.g. already released) (sigh), so double-check here ... */
-  if (!g_slist_find (ffmpegdec->opaques, ffmpegdec->picture->opaque)) {
-    GST_ERROR_OBJECT (ffmpegdec, "invalid picture opaque");
-    goto beach;
-  }
-
   /* get the output picture timing info again */
   out_dframe = ffmpegdec->picture->opaque;
   out_frame = gst_video_codec_frame_ref (out_dframe->frame);
@@ -1261,6 +1280,8 @@ gst_ffmpegviddec_video_frame (GstFFMpegVidDec * ffmpegdec,
       ffmpegdec->picture->repeat_pict);
   GST_DEBUG_OBJECT (ffmpegdec, "interlaced_frame:%d (current:%d)",
       ffmpegdec->picture->interlaced_frame, ffmpegdec->ctx_interlaced);
+  GST_DEBUG_OBJECT (ffmpegdec, "corrupted frame: %d",
+      ! !(ffmpegdec->picture->flags & AV_FRAME_FLAG_CORRUPT));
 
   if (G_UNLIKELY (ffmpegdec->picture->interlaced_frame !=
           ffmpegdec->ctx_interlaced)) {
@@ -1277,6 +1298,10 @@ gst_ffmpegviddec_video_frame (GstFFMpegVidDec * ffmpegdec,
   if (G_UNLIKELY (*ret != GST_FLOW_OK))
     goto no_output;
 
+  /* Mark corrupted frames as corrupted */
+  if (ffmpegdec->picture->flags & AV_FRAME_FLAG_CORRUPT)
+    GST_BUFFER_FLAG_SET (out_frame->output_buffer, GST_BUFFER_FLAG_CORRUPTED);
+
   if (ffmpegdec->ctx_interlaced) {
     /* set interlaced flags */
     if (ffmpegdec->picture->repeat_pict)
@@ -1365,11 +1390,11 @@ negotiation_error:
 
 static gint
 gst_ffmpegviddec_frame (GstFFMpegVidDec * ffmpegdec,
-    guint8 * data, guint size, gint * got_data, GstVideoCodecFrame * frame,
+    guint8 * data, guint size, gint * have_data, GstVideoCodecFrame * frame,
     GstFlowReturn * ret)
 {
   GstFFMpegVidDecClass *oclass;
-  gint have_data = 0, len = 0;
+  gint len = 0;
 
   if (G_UNLIKELY (ffmpegdec->context->codec == NULL))
     goto no_codec;
@@ -1381,27 +1406,16 @@ gst_ffmpegviddec_frame (GstFFMpegVidDec * ffmpegdec,
 
   oclass = (GstFFMpegVidDecClass *) (G_OBJECT_GET_CLASS (ffmpegdec));
 
-  len = gst_ffmpegviddec_video_frame (ffmpegdec, data, size, frame, ret);
+  len =
+      gst_ffmpegviddec_video_frame (ffmpegdec, data, size, have_data, frame,
+      ret);
 
-  if (frame && frame->output_buffer)
-    have_data = 1;
-
-  if (len < 0 || have_data < 0) {
+  if (len < 0) {
     GST_WARNING_OBJECT (ffmpegdec,
         "avdec_%s: decoding error (len: %d, have_data: %d)",
-        oclass->in_plugin->name, len, have_data);
-    *got_data = 0;
-    goto beach;
-  }
-  if (len == 0 && have_data == 0) {
-    *got_data = 0;
-    goto beach;
+        oclass->in_plugin->name, len, *have_data);
   }
 
-  /* this is where I lost my last clue on ffmpeg... */
-  *got_data = 1;
-
-beach:
   return len;
 
   /* ERRORS */
@@ -1418,10 +1432,13 @@ gst_ffmpegviddec_drain (GstFFMpegVidDec * ffmpegdec)
 {
   GstFFMpegVidDecClass *oclass;
 
+  if (!ffmpegdec->opened)
+    return;
+
   oclass = (GstFFMpegVidDecClass *) (G_OBJECT_GET_CLASS (ffmpegdec));
 
   if (oclass->in_plugin->capabilities & CODEC_CAP_DELAY) {
-    gint have_data, len, try = 0;
+    gint have_data, len;
 
     GST_LOG_OBJECT (ffmpegdec,
         "codec has delay capabilities, calling until ffmpeg has drained everything");
@@ -1430,9 +1447,9 @@ gst_ffmpegviddec_drain (GstFFMpegVidDec * ffmpegdec)
       GstFlowReturn ret;
 
       len = gst_ffmpegviddec_frame (ffmpegdec, NULL, 0, &have_data, NULL, &ret);
-      if (len < 0 || have_data == 0)
-        break;
-    } while (try++ < 10);
+
+    } while (len >= 0 && have_data == 1);
+    avcodec_flush_buffers (ffmpegdec->context);
   }
 }
 
@@ -1516,7 +1533,7 @@ gst_ffmpegviddec_handle_frame (GstVideoDecoder * decoder,
       break;
     }
 
-    if (len == 0 && !have_data) {
+    if (len == 0 && have_data == 0) {
       /* nothing was decoded, this could be because no data was available or
        * because we were skipping frames.
        * If we have no context we must exit and wait for more data, we keep the
@@ -1591,8 +1608,6 @@ gst_ffmpegviddec_stop (GstVideoDecoder * decoder)
   if (ffmpegdec->output_state)
     gst_video_codec_state_unref (ffmpegdec->output_state);
   ffmpegdec->output_state = NULL;
-  g_slist_free (ffmpegdec->opaques);
-  ffmpegdec->opaques = NULL;
 
   ffmpegdec->ctx_width = 0;
   ffmpegdec->ctx_height = 0;
@@ -1635,7 +1650,7 @@ gst_ffmpegviddec_decide_allocation (GstVideoDecoder * decoder, GstQuery * query)
   GstBufferPool *pool;
   guint size, min, max;
   GstStructure *config;
-  gboolean have_videometa, have_alignment;
+  gboolean have_videometa, have_alignment, update_pool = FALSE;
   GstAllocator *allocator = NULL;
   GstAllocationParams params = { 0, 15, 0, 0, };
 
@@ -1654,6 +1669,22 @@ gst_ffmpegviddec_decide_allocation (GstVideoDecoder * decoder, GstQuery * query)
 
   gst_query_parse_nth_allocation_pool (query, 0, &pool, &size, &min, &max);
 
+  /* Don't use pool that can't grow, as we don't know how many buffer we'll
+   * need, otherwise we may stall */
+  if (max != 0 && max < REQUIRED_POOL_MAX_BUFFERS) {
+    gst_object_unref (pool);
+    pool = gst_video_buffer_pool_new ();
+    max = 0;
+    update_pool = TRUE;
+
+    /* if there is an allocator, also drop it, as it might be the reason we
+     * have this limit. Default will be used */
+    if (allocator) {
+      gst_object_unref (allocator);
+      allocator = NULL;
+    }
+  }
+
   config = gst_buffer_pool_get_config (pool);
   gst_buffer_pool_config_set_params (config, state->caps, size, min, max);
   /* we are happy with the default allocator but we would like to have 16 bytes
@@ -1684,8 +1715,8 @@ gst_ffmpegviddec_decide_allocation (GstVideoDecoder * decoder, GstQuery * query)
     avcodec_align_dimensions2 (ffmpegdec->context, &width, &height,
         linesize_align);
     edge =
-        ffmpegdec->context->
-        flags & CODEC_FLAG_EMU_EDGE ? 0 : avcodec_get_edge_width ();
+        ffmpegdec->
+        context->flags & CODEC_FLAG_EMU_EDGE ? 0 : avcodec_get_edge_width ();
     /* increase the size for the padding */
     width += edge << 1;
     height += edge << 1;
@@ -1741,6 +1772,9 @@ gst_ffmpegviddec_decide_allocation (GstVideoDecoder * decoder, GstQuery * query)
   /* and store */
   gst_buffer_pool_set_config (pool, config);
 
+  if (update_pool)
+    gst_query_set_nth_allocation_pool (query, 0, pool, size, min, max);
+
   gst_object_unref (pool);
   if (allocator)
     gst_object_unref (allocator);
@@ -1791,6 +1825,9 @@ gst_ffmpegviddec_set_property (GObject * object,
     case PROP_MAX_THREADS:
       ffmpegdec->max_threads = g_value_get_int (value);
       break;
+    case PROP_OUTPUT_CORRUPT:
+      ffmpegdec->output_corrupt = g_value_get_boolean (value);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -1819,6 +1856,9 @@ gst_ffmpegviddec_get_property (GObject * object,
     case PROP_MAX_THREADS:
       g_value_set_int (value, ffmpegdec->max_threads);
       break;
+    case PROP_OUTPUT_CORRUPT:
+      g_value_set_boolean (value, ffmpegdec->output_corrupt);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -1913,7 +1953,11 @@ gst_ffmpegviddec_register (GstPlugin * plugin)
     }
 
     /* construct the type */
-    plugin_name = g_strdup ((gchar *) in_plugin->name);
+    if (!strcmp (in_plugin->name, "hevc")) {
+      plugin_name = g_strdup ("h265");
+    } else {
+      plugin_name = g_strdup ((gchar *) in_plugin->name);
+    }
     g_strdelimit (plugin_name, NULL, '_');
     type_name = g_strdup_printf ("avdec_%s", plugin_name);
     g_free (plugin_name);
@@ -1936,6 +1980,7 @@ gst_ffmpegviddec_register (GstPlugin * plugin)
       case AV_CODEC_ID_MPEG4:
       case AV_CODEC_ID_MSMPEG4V3:
       case AV_CODEC_ID_H264:
+      case AV_CODEC_ID_HEVC:
       case AV_CODEC_ID_RV10:
       case AV_CODEC_ID_RV20:
       case AV_CODEC_ID_RV30:
