@@ -339,7 +339,9 @@ void av_des_mac(AVDES *d, uint8_t *dst, const uint8_t *src, int count) {
 #ifdef TEST
 #include <stdlib.h>
 #include <stdio.h>
-#include "libavutil/time.h"
+
+#include "time.h"
+
 static uint64_t rand64(void) {
     uint64_t r = rand();
     r = (r << 32) | rand();
